<?php
namespace Drupal\test_by_contenttype;

/**
 * Generic CURL function
 *
 * PHP version 7
 *
 *
 * @category HTML
 * @package  TestByContenttype
 * @author   Zef Oudendorp <zef@kees-tm.nl>
 * @license  MIT
 * @link     https://packagist.org/packages/keestm/test-by-contenttype
 */
class Curler
{
 
    /**
     * Valide HTML using W3C validation API
     *
     * @param string  $url we're about to cURL
     * @param string  $data_string data to post
     * @param boolean $get_http_status to only return the HTTP status of the url
     *
     * @return mixed
     *
     */
    public function do($url, $data_string = "", $get_http_status = "")
    {
        $curl = curl_init();
        curl_setopt($curl, CURLOPT_URL, $url);
        curl_setopt($curl, CURLOPT_HEADER, ($get_http_status? true:false)); //still returning header?
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        //Disable SSL verification!
        curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, false);
        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
        //post fields if present
        if (!empty($data_string)) {
            $http_header = array(
                'Content-Type: application/json',
                'Content-Length: ' . strlen($data_string)
            );
            curl_setopt($curl, CURLOPT_POST, true);
            curl_setopt($curl, CURLOPT_POSTFIELDS, $data_string);
            curl_setopt($curl, CURLOPT_HTTPHEADER, $http_header);
        }
        if ($get_http_status) {
            curl_setopt($curl, CURLOPT_NOBODY, true);
        }
        //Expressionengine uses these options
        curl_setopt($curl, CURLOPT_FOLLOWLOCATION, true);
        curl_setopt($curl, CURLOPT_VERBOSE, true);
        curl_setopt($curl, CURLOPT_USERAGENT, 'Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.52 Safari/537.17');
        $return_data = curl_exec($curl);
        // Check for errors and display the error message
        if ($errno = curl_errno($curl) || $return_data === false) {
            $error_message = curl_strerror($errno);
            echo "cURL error ({$errno}):\n {$error_message} for ".$url."<br />";
        }
        if ($get_http_status) {
            $return_data = curl_getinfo($curl, CURLINFO_HTTP_CODE);
        }
        curl_close($curl);
        return $return_data;
    }
}